package com.scbfkj.uni.process;

import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import org.apache.activemq.ActiveMQConnectionFactory;

import javax.jms.*;
import java.util.*;

public class ActiveMQ {


    private static Map<String, Session> sessionMap = new HashMap<>();


    /**
     * 采集
     *
     * @return 读取内容信息
     */
    public static Map<String, Object> readMessage(String sourceObjectName, String connectConfig, String pollNumber) {
        if (Objects.isNull(sourceObjectName) || Objects.isNull(connectConfig)) {
            return UniReturnUtil.fail(Objects.isNull(sourceObjectName) ? "队列为空" : "连接字符串为空");
        }
        try {

            int max = pollNumber == null ? 1 : Integer.parseInt(pollNumber);
            if (max < 0) {
                max = Math.abs(max);
            }
            Session session;
            session = sessionMap.get(connectConfig);
            if (session == null) {
                session = createSession(connectConfig);
            }
            try (MessageConsumer consumer = session.createConsumer(session.createQueue(sourceObjectName))) {

                ArrayList<String> result = new ArrayList<>();

                for (int i = 0; i < max; i++) {
                    TextMessage receive = (TextMessage) consumer.receive(100);
                    if (Objects.nonNull(receive)) {
                        result.add(receive.getText());
                        session.commit();
                    } else {
                        break;
                    }
                }
                return UniReturnUtil.success(result);
            }
        } catch (JMSException e) {
            return UniReturnUtil.fail(e.getMessage());
        }
    }

    private static Session createSession(String connectionStr) throws JMSException {
        Map<?, ?> connectionConfig = DataFormatUtil.toMap(connectionStr);

        String host = connectionConfig.get("host").toString();
        Object username = connectionConfig.get("username");
        Object password = connectionConfig.get("password");
        ConnectionFactory connectionFactory = new ActiveMQConnectionFactory(username == null ? null : username.toString(), password == null ? null : password.toString(), host);
        Connection connection = connectionFactory.createConnection();
        connection.start();
        return connection.createSession(true, Session.AUTO_ACKNOWLEDGE);

    }


    /**
     * 数据发送
     *
     * @return 发送结果信息
     */
    public static Map<String, Object> sendMethod(String sourceObjectName, List<Object> data, String connectConfig) {
        if (Objects.isNull(sourceObjectName) || Objects.isNull(data)) {
            return UniReturnUtil.fail(Objects.isNull(sourceObjectName) ? ("队列为空 " + data) : ("数据内容为空 " + data));
        }
        try {
            Session session;
            session = sessionMap.get(connectConfig);
            if (session == null) {
                session = createSession(connectConfig);
            }
            try (MessageProducer producer = session.createProducer(session.createQueue(sourceObjectName))) {

                for (Object datum : data) {
                    //创建一个文本消息
                    if (Objects.nonNull(datum)) {
                        TextMessage textMessage = session.createTextMessage(datum.toString());
                        producer.send(textMessage);//生产者发送消息
                    }
                }
                session.commit();//会话提交
                return UniReturnUtil.success(null);
            }
        } catch (JMSException e) {
            return UniReturnUtil.fail(e.getMessage());
        }
    }

}
