package com.scbfkj.uni.process;

import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.springframework.util.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.zip.GZIPInputStream;

import static org.apache.commons.net.ftp.FTP.BINARY_FILE_TYPE;

public class FTP {
    private static final int BUFFER_SIZE = 4096;

    public static Map<String, Object> update(String connectionConfig, String targetPath, String sourcePath) {
        return null;
    }


    /**
     * 设置ftp客户端参数并创建ftp客户端
     *
     * @param config ftp连接配置信息
     * @return ftp客户端
     */
    private static FTPClient createFtpClient(Map<String, Object> config) throws Exception {
        FTPClient ftp = new FTPClient();
        int port = Objects.nonNull(config.get("port")) ? Integer.parseInt(Objects.toString(config.get("port"))) : 21;
        ftp.connect(Objects.toString(config.get("host")), port);
        //common-net的ftpclient默认是使用ASCII_FILE_TYPE，文件会经过ASCII编码转换，所以可能会造成文件损坏
        ftp.login(Objects.toString(config.get("username")), Objects.toString(config.get("password")));//登录
        String timeOutStr = "timeOut";
        //120秒
        int timeOut = Objects.nonNull(config.get(timeOutStr)) ? Integer.parseInt(Objects.toString(config.get(timeOutStr))) : (1000 * 120);
        ftp.setConnectTimeout(timeOut);
        ftp.setDataTimeout(Duration.ofSeconds(timeOut));
        //每次读取文件流时缓存数组的大小
        ftp.setBufferSize(1024 * 1024 * 10);
        // 取消服务器获取自身Ip地址和提交的host进行匹配
        ftp.setRemoteVerificationEnabled(false);
        //设置ftp编码
        String encoding = Objects.nonNull(config.get("encoding")) ? config.get("encoding").toString() : "UTF-8";
        ftp.setControlEncoding(encoding);
        ftp.setFileType(BINARY_FILE_TYPE);//设置文件编码类型为二进制文件，
        return ftp;
    }

    /**
     * @param sourcePath       组装好的文件名称
     * @param connectionConfig 连接fpt的连接信息
     * @return
     */
    public static Map<String, Object> download(String connectionConfig, String sourcePath) {

        Map connectionConfigMaps = DataFormatUtil.toMap(connectionConfig);
        FTPClient ftp = null;
        try {
            List<Object> fileList = new ArrayList<>();
            fileList.add(sourcePath);
            String basePath = Objects.toString(connectionConfigMaps.get("basePath")); //根路径
            ftp = createFtpClient(connectionConfigMaps);
            if (!FTPReply.isPositiveCompletion(ftp.getReplyCode())) {
                System.err.println("未连接到FTP，用户名或密码错误。");
                ftp.logout();
                return UniReturnUtil.fail("未连接到FTP，用户名或密码错误。");
            }
            if (StringUtils.hasLength(basePath)) {
                ftp.changeWorkingDirectory(basePath);
            }
            String rootPath = ftp.printWorkingDirectory(); //ftp根目录
            FTPFile[] ftpFiles = ftp.listFiles(rootPath);
            if (ftpFiles.length == 0) {
                System.err.println("没有找到文件");
                ftp.logout();
                return UniReturnUtil.fail("没有找到文件");
            }
            List<File> files = new ArrayList<>();
            for (FTPFile ftpFile : ftpFiles) {
                String tpFileName = ftpFile.getName();
                System.err.println(tpFileName);
                if (fileList.contains(tpFileName)) {
                    String path = System.getProperty("user.dir") + File.separator + "ftp" + File.separator + ftpFile.getName();
                    File targetFile = new File(path);
                    files.add(targetFile);
                    try (FileOutputStream fileOutputStream = new FileOutputStream(targetFile);) {
                        ftp.retrieveFile(ftpFile.getName(), fileOutputStream);
                    } catch (Exception e) {
                        System.err.println("数据下载异常".concat(UniReturnUtil.getMessage(e)));
                    }
                }
            }
            ftp.logout();//退出ftp
            List<String> fileContent = new ArrayList<>();
            for (File ftpFile : files) {
                String path = System.getProperty("user.dir") + File.separator + "ftp" + File.separator + ftpFile.getName();
                File targetFile = new File(path);
                if (targetFile.exists()) {
                    try (GZIPInputStream gzipInputStream = new GZIPInputStream(Files.newInputStream(targetFile.toPath()));
                         ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    ) {
                        byte[] buf = new byte[BUFFER_SIZE];
                        int len;
                        while ((len = gzipInputStream.read(buf, 0, BUFFER_SIZE)) != -1) {
                            baos.write(buf, 0, len);
                        }
                        fileContent.add(baos.toString(StandardCharsets.UTF_8));
                    } catch (Exception e) {
                        System.err.println("数据解压异常".concat(UniReturnUtil.getMessage(e)));
                    }
                }
            }
            return UniReturnUtil.success(fileContent);
        } catch (Exception e) {
            return UniReturnUtil.fail("fpt读取文件异常".concat(UniReturnUtil.getMessage(e)));
        } finally {
            try {
                if (null != ftp) {
                    ftp.disconnect();
                }
            } catch (Exception ex) {
                System.out.println("ftp关闭时异常:".concat(ex.getMessage()));
            }
        }
    }
}
