package com.scbfkj.uni.process;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.GetResponse;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeoutException;

public class RabbitMQ {

    private final static Map<String, ConnectionFactory> factories = new HashMap<>();


    public static ConnectionFactory createConnectFactory(String connectionStr) {
        ConnectionFactory connectionFactory = factories.get(connectionStr);
        if (Objects.nonNull(connectionFactory)) return connectionFactory;

        Map<?, ?> connectConfig = DataFormatUtil.toMap(connectionStr);

        connectionFactory = new ConnectionFactory();
        connectionFactory.setHost(connectConfig.get("host").toString());
        connectionFactory.setUsername(connectConfig.get("username").toString());
        connectionFactory.setPassword(connectConfig.get("password").toString());
        connectionFactory.setPort(Integer.parseInt(connectConfig.get("port").toString()));
        Object virtualHost = connectConfig.get("virtualHost");
        if (Objects.nonNull(virtualHost)) connectionFactory.setVirtualHost(virtualHost.toString());
        connectionFactory.setConnectionTimeout(3000);
        return connectionFactory;
    }

    public static Map<String, Object> sendMessage(String connectionConfig, String exchangeName, String data) throws Exception {

        ConnectionFactory connectFactory = createConnectFactory(connectionConfig);
        try (Connection connection = connectFactory.newConnection()) {
            try (Channel channel = connection.createChannel()) {
//                routingKey = (routingKey == null) ? queueName : routingKey;
//                exchangeName = (null == exchangeName) ? "" : exchangeName;
//                AMQP.BasicProperties properties = new AMQP.BasicProperties.Builder().deliveryMode(2).contentEncoding("UTF-8").build();
                //第一个参数是exchange参数，如果是为空字符串，那么就会发送到(AMQP default)默认的exchange，而且routingKey
                //便是所要发送到的队列名
                //todo 如果rabbitmq的服务端为绑定路由，那么写入不进行
                //todo exchangeName为“” 则rabbitmq服务端不需要配置
                //todo exchangeName为不为"" 则rabbitmq必须绑定
                channel.exchangeDeclare(exchangeName, "fanout");
                channel.basicPublish(exchangeName, "/", null, data.getBytes(StandardCharsets.UTF_8));
            } catch (Exception e) {
                return UniReturnUtil.fail(UniReturnUtil.getMessage(e));
            }
        } catch (IOException | TimeoutException e) {
            return UniReturnUtil.fail(UniReturnUtil.getMessage(e));
        }

        return null;
    }

    public static Map<String, Object> receptionMessage(String connectConfig, String routingKey, String exchangeName) throws Exception {

        Map<String, Object> connectConfigMaps = (Map<String, Object>) DataFormatUtil.toMap(connectConfig);
        Object arguments = connectConfigMaps.get("arguments");
//        Object prefetchCount = connectConfigMaps.get("prefetchCount");
        try (Connection connection = createConnectFactory(connectConfig).newConnection(); Channel currentChannel = connection.createChannel();) {
            //创建消息通道
            currentChannel.queueDeclare(exchangeName, true, false, false, Objects.isNull(arguments) ? null : ((Map<String, Object>) arguments));
            currentChannel.basicQos(1); //服务端在同一时刻只发送1条数据
            GetResponse getResponse = currentChannel.basicGet(exchangeName, true);
            String repBody = null;
            if (null != getResponse) {
                repBody = new String(getResponse.getBody());
            }
            return UniReturnUtil.success(repBody);
        } catch (IOException | TimeoutException e) {
            return UniReturnUtil.fail(UniReturnUtil.getMessage(e));
        }
    }
}
