package com.scbfkj.uni.service;

import com.scbfkj.uni.library.DataAliasGetUtil;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.library.script.DatabaseScriptUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.system.Config;
import com.scbfkj.uni.system.ScheduleUtil;

import java.time.LocalDateTime;
import java.util.*;

public class ControlService {


    public static void stopServiceByContainerCode() throws Exception {

//        查询服务类型为主动采集的服务
        String queryServiceInfos = "select * from serviceinfo where containercode=? ";
        List<Map<String, Object>> serviceInfos = DataBase.query(Config.centerConnectionStr, queryServiceInfos, Collections.singletonList(new Object[]{Config.containerCode}));
//        循环启动
        for (Map<String, Object> serviceInfo : serviceInfos) {
            stop(serviceInfo.get("serviceid").toString());
        }
    }

    public static void startServiceByContainerCode() throws Exception {

//        查询服务类型为主动采集的服务
        String queryServiceInfos = "select * from serviceinfo where containercode=? ";
        List<Map<String, Object>> serviceInfos = DataBase.query(Config.centerConnectionStr, queryServiceInfos, Collections.singletonList(new Object[]{Config.containerCode}));
//        循环启动
        for (Map<String, Object> serviceInfo : serviceInfos) {
            start(serviceInfo);
        }
    }

    /**
     * 服务启动 主要用于前端或者第三方请求
     *
     * @param inData
     * @return
     */
    public static Map<String, Object> start(Map<String, Object> inData) throws Exception {
        Optional<String> serviceidOpt = DataAliasGetUtil.getValue("serviceid", inData);
        if (serviceidOpt.isEmpty()) {
            throw new RuntimeException("服务编号不能为空");
        }
        String serviceId = serviceidOpt.get();
        List<Map<String, Object>> serviceInfoList = null;
            serviceInfoList = DataBase.query(Config.centerConnectionStr, "select * from serviceinfo where serviceid=?", Collections.singletonList(new Object[]{serviceId}));

        if (serviceInfoList.isEmpty()) {
            return UniReturnUtil.fail("服务%s没有配置".formatted(serviceId));
        }
        Map<String, Object> serviceInfo = serviceInfoList.get(0);
        Object cronexpress = serviceInfo.get("cronexpress");
        if (Objects.isNull(cronexpress)) {
//            设置服务状态为运行中

            updateServiceState(serviceId, "1");
            return UniReturnUtil.success(null);
        } else {
            //启动定时任务：
            boolean start = ScheduleUtil.start(serviceId);
            if (start) {

                updateServiceState(serviceId, "1");
                return UniReturnUtil.success("启动成功");
            } else {
                return UniReturnUtil.fail("启动失败");
            }
        }
    }


    /**
     * 服务停止
     *
     * @param serviceId
     * @return
     */
    public static Map<String, Object> stop(String serviceId) throws Exception {
        boolean cancel = ScheduleUtil.cancel(serviceId);
        updateServiceState(serviceId,"0");
        return UniReturnUtil.success("停止" + (cancel ? "成功" : "失败"));
    }

    private static void updateServiceState(String serviceId, String state) throws Exception {
        DatabaseScriptUtil.exec(Config.centerConnectionStr, "servicestate", new ArrayList<>() {{
            add(new HashMap<>() {{
                put("value", new HashMap<>() {{
                    put("serviceid", serviceId);
                    put("starttime", DataFormatUtil.toString(LocalDateTime.now()));
                    put("containercode", Config.containerCode);
                    put("runstate", state);

                }});
                put("filter", new HashMap<>() {{
                    put("serviceid", serviceId);
                    put("containercode", Config.containerCode);

                }});

            }});
        }}, "6", null, null);
    }

    /**
     * 用用户启停定时任务 服务类型为4的服务
     *
     * @param params
     * @param statue
     * @return
     * @throws Exception
     */

    public static Map<String, Object> startOrStop(Map<String, Object> params, String statue, boolean all) throws Exception {
        Optional<String> serviceIdOpt = DataAliasGetUtil.getValue("serviceid", params);
        if (serviceIdOpt.isEmpty() && !all) {
            return UniReturnUtil.fail("服务ID不能为空");
        } else if (serviceIdOpt.isPresent()) {

            String serviceId = serviceIdOpt.get();
            Map<String, Object> stop = stop(serviceId);
            if (statue.equals("0")) {
                return stop;
            }
            List<Map<String, Object>> serviceTypeData = DataBase.query(Config.centerConnectionStr, "select serviceType from serviceinfo where serviceID=? and containercode=? and servicetype='4'", Collections.singletonList(new Object[]{serviceId, Config.containerCode}));
            if (serviceTypeData.isEmpty()) {
                throw new RuntimeException("没有找到服务配置");
            }
            return start(new HashMap<>() {{
                put("serviceid", serviceId);
            }});
        } else {
            if (Objects.equals(statue, "0")) {
                stopServiceByContainerCode();

                return UniReturnUtil.success(true);
            } else {
                startServiceByContainerCode();
                return UniReturnUtil.success(true);
            }
        }
    }
}
